@@ -1,6 +1,7 @@
 
 import collections
 import copy
+import glob
 import fnmatch
 import io
 import itertools
@@ -33,6 +34,18 @@ __all__ = [
 ]
 
 
+SLACKTIVATE_SORT_NEWEST = "newest"
+
+SLACKTIVATE_SORT_OLDEST = "oldest"
+
+SLACKTIVATE_DEFAULT_SORT = SLACKTIVATE_SORT_NEWEST
+
+DEFAULT_EMAIL_INDEX = 0
+
+EMAIL_FIELD_NAME = "email"  # used only for alternate_emails feature
+ALTERNATE_EMAIL_FIELD_NAME = "alternate_emails"
+
+
 class SlacktivateJSONEncoder(json.JSONEncoder):
 
     def default(self, obj):
@@ -63,9 +76,9 @@ class SlacktivateConfigSection(collections.UserDict):
 
     def __init__(self, value, **kwargs):
         super().__init__(value, **kwargs)
-        self._validate_fields()
+        self._validate_fields(**kwargs)
 
-    def _validate_fields(self):
+    def _validate_fields(self, **kwargs):
 
         # check required fields
         if self._required is not None:
@@ -113,9 +126,12 @@ class UserSourceException(ValueError):
         super().__init__(*args, **kwargs)
         self._message = message
 
+    def __str__(self):
+        return self.message
+
     @property
     def message(self):
-        return self._message
+        return self._message if self._message is not None else ""
 
 
 class UserSourceConfig(SlacktivateConfigSection):
@@ -128,18 +144,57 @@ class UserSourceConfig(SlacktivateConfigSection):
     # }
 
     _required = ["type", ["file", "contents"]]
-    _optional = ["fields", "key", "filter"]
+    _optional = ["fields", "key", "filter", "sort"]
+    _source_name = None
 
     def __init__(self, value):
         super().__init__(value)
+        self._validate_fields(vars=None)
+
+    def _validate_fields(
+            self,
+            vars: typing.Optional[typing.Dict[str, str]] = None,
+            **kwargs,
+    ) -> typing.NoReturn:
+
+        # first validate parent logic
+        super()._validate_fields(vars=vars, **kwargs)
 
         if "file" in self:
-            if not os.path.exists(self.get("file")):
-                raise UserSourceException(
-                    "configuration file problem: user source '{}' cannot be found\n(pwd: '{}')".format(
-                        self.get("file"),
-                        os.getcwd()),
-                )
+
+            # first determine if we need the 'vars', and only continue if we don't need
+            # it or we need it and it is provided to this function --- yes the condition in
+            # the if is redundant, but intentionally so)
+
+            file_str = self.get("file")
+            file_str_fields = slacktivate.input.helpers.find_jinja2_template_fields(file_str)
+
+            if "vars" not in file_str_fields or ("vars" in file_str_fields and vars is not None):
+
+                if len(file_str_fields) > 0:
+                    file_str = slacktivate.input.helpers.render_jinja2(
+                        jinja2_pattern=file_str,
+                        data=None,
+                        vars=vars,
+                    )
+
+                # next "file" could be either a path, or a glob expression;
+                # we raise an exception if the path does not exist and the
+                # expression does not glob
+
+                if not os.path.exists(file_str) and len(glob.glob(file_str)) == 0:
+                    # looks like debug code right??
+                    # print("<<<{}>>>".format(file_str))
+                    # print("<<<{}>>>".format(glob.glob(file_str)))
+                    # print("<<<{}>>>".format(len(glob.glob(file_str))))
+                    # print("configuration file problem: user source '{}' cannot be found\n(pwd: '{}')".format(
+                    #     self.get("file"),
+                    #     os.getcwd()))
+                    raise UserSourceException(
+                        "configuration file problem: user source '{}' cannot be found\n(pwd: '{}')".format(
+                            self.get("file"),
+                            os.getcwd()),
+                    )
 
         if "key" in self:
             if not slacktivate.input.helpers.parseable_jinja2(self.get("key")):
@@ -148,13 +203,44 @@ class UserSourceConfig(SlacktivateConfigSection):
                         self.get("file")),
                 )
 
-    def _load(self) -> list:
+    def _load(
+            self,
+            vars: typing.Optional[typing.Dict[str, str]],
+    ) -> list:
+
+        # revalidate fields with 'vars' in case the validation of filename was skipped
+        self._validate_fields(vars=vars)
 
         raw_data = None
 
         # getting the data
         if "file" in self:
-            raw_data = open(self.get("file")).read()
+            file_str = slacktivate.input.helpers.render_jinja2(
+                jinja2_pattern=self.get("file"),
+                data=None,
+                vars=vars,
+            )
+
+            files = glob.glob(file_str)
+
+            # by default, sort is in increasing order of mtime
+            # but if we want newest, we want the largest mtime first,
+            # so that is reverse
+            reverse_sort = (
+                self.get("sort", SLACKTIVATE_DEFAULT_SORT) == SLACKTIVATE_SORT_NEWEST
+            )
+
+            files.sort(
+                key=os.path.getmtime,
+                reverse=reverse_sort,
+            )
+
+            # guaranteed to exist otherwise we would have raised an exception
+            # above when validating parameters
+            file = files[0]
+            self._source_name = file
+
+            raw_data = open(file).read()
 
         elif "contents" in self:
             raw_data = self.get("contents")
@@ -169,11 +255,48 @@ class UserSourceConfig(SlacktivateConfigSection):
             data = yaml.load(raw_data)
 
         elif self.get("type") == "csv":
-            data = comma.load(raw_data)
+            data = list(map(dict, comma.load(raw_data, force_header=True)))
 
         return data
 
-    def _post_process(self, value: typing.Union[list, dict]) -> list:
+    def _post_process(
+            self,
+            data: typing.Union[list, dict],
+            vars: typing.Optional[typing.Dict[str, str]],
+            alternate_emails: typing.Optional[typing.Dict[str, typing.List[str]]] = None,
+    ) -> list:
+
+        if alternate_emails is not None:
+            # this is a feature to make sure that, when a user is known
+            # with an alias, that alias can be prioritized (or deprioritized)
+            # centrally through the `alternate_emails` mechanisms
+            # { ...
+            #     "user1@domain.com": ["user1@domain.com", "user1.alias@domain.com"],
+            # ... }
+            # => replace emails
+            iterable = slacktivate.input.helpers.iterable_from_list_or_dict(data)
+            for user_row in iterable:
+                # only place where EMAIL_FIELD_NAME is needed
+                email = user_row.get(EMAIL_FIELD_NAME)
+                if email is None:
+                    continue
+
+                ae_lookup = alternate_emails.get(email)
+                ae_lookup = ae_lookup or alternate_emails.get(email.lower())
+                if ae_lookup is None:
+                    continue
+
+                # shouldn't happen but let's see
+                if type(ae_lookup) is str:
+                    ae_lookup = [ae_lookup]
+
+                # ae_lookup should be a list
+                if email not in ae_lookup:
+                    ae_lookup += [email]
+
+                user_row[EMAIL_FIELD_NAME] = ae_lookup[DEFAULT_EMAIL_INDEX]
+                user_row[ALTERNATE_EMAIL_FIELD_NAME] = ae_lookup
+
 
         # the 'key' field is to reindex the database
         if "key" in self and self.get("key") is not None:
@@ -181,54 +304,87 @@ class UserSourceConfig(SlacktivateConfigSection):
             key_pattern = self.get("key")
 
             # reindex data
-            value = slacktivate.input.helpers.reindex_user_data(
-                user_data=value,
+            data = slacktivate.input.helpers.reindex_user_data(
+                user_data=data,
                 key=key_pattern,
             )
+            iterable = slacktivate.input.helpers.iterable_from_list_or_dict(data)
 
             # store the key
-            for record in value.values():
+            for record in iterable:
                 record["key"] = key_pattern
 
         # create additional programmable fields
         if "fields" in self and self.get("fields") is not None:
 
             def __expand_record(record):
-                new_fields = {
-                    field_name: slacktivate.input.helpers.render_jinja2(
-                        jinja2_pattern=field_pattern,
+                new_fields = {}
+
+                def formatter(pattern):
+                    return slacktivate.input.helpers.render_jinja2(
+                        jinja2_pattern=pattern,
                         data=record,
+                        vars=vars,
                     )
-                    for field_name, field_pattern in self.get("fields").items()
-                }
+
+                for field_name, field_pattern in self.get("fields").items():
+
+                    if isinstance(field_pattern, str):
+                        new_fields[field_name] = formatter(field_pattern)
+
+                    elif isinstance(field_pattern, list):
+
+                        # retrieve record's value for this field, if it exists
+                        current_field_value = record.get(field_name, list())
+                        if isinstance(current_field_value, str):
+                            current_field_value = [current_field_value]
+
+                        # format current field patterns:
+                        new_values = list(map(formatter, field_pattern))
+
+                        # assign field
+                        new_fields[field_name] = current_field_value + new_values
+
+                # new_fields = {
+                #     field_name: slacktivate.input.helpers.render_jinja2(
+                #         jinja2_pattern=field_pattern,
+                #         data=record,
+                #         vars=vars,
+                #     )
+                #     for field_name, field_pattern in self.get("fields").items()
+                # }
                 record.update(new_fields)
                 return record
 
-            if issubclass(type(value), list) or issubclass(type(value), collections.UserList):
-                value = [
+            if issubclass(type(data), list) or issubclass(type(data), collections.UserList):
+                data = [
                     __expand_record(record)
-                    for record in value
+                    for record in data
                 ]
 
-            if issubclass(type(value), dict) or issubclass(type(value), collections.UserDict):
-                value = {
+            if issubclass(type(data), dict) or issubclass(type(data), collections.UserDict):
+                data = {
                     key: __expand_record(record)
-                    for key, record in value.items()
+                    for key, record in data.items()
                 }
 
         # refilter data
         if "filter" in self and self.get("fields") is not None:
-            value = slacktivate.input.helpers.refilter_user_data(
-                user_data=value,
+            data = slacktivate.input.helpers.refilter_user_data(
+                user_data=data,
                 filter_query=self.get("filter"),
                 reindex=self.get("key") is not None,
             )
 
-        return value
+        return data
 
-    def load(self) -> list:
-        data = self._load()
-        data = self._post_process(data)
+    def load(
+            self,
+            vars: typing.Optional[typing.Dict[str, str]],
+            alternate_emails: typing.Optional[typing.Dict[str, typing.List[str]]] = None,
+    ) -> list:
+        data = self._load(vars=vars)
+        data = self._post_process(data, vars=vars, alternate_emails=alternate_emails)
         return data
 
 
@@ -252,7 +408,8 @@ class UserGroupConfig(SlacktivateConfigSection):
 
     def compute(
             self,
-            users: typing.Union[list, dict]
+            users: typing.Union[list, dict],
+            vars: typing.Optional[typing.Dict[str, str]],
     ) -> typing.List["UserGroupConfig"]:
 
         target_users = users
@@ -271,6 +428,7 @@ class UserGroupConfig(SlacktivateConfigSection):
             group_name = slacktivate.input.helpers.render_jinja2(
                 jinja2_pattern=self.get("name"),
                 data=user,
+                vars=vars,
             )
 
             subgroup_users[group_name] = subgroup_users.get(group_name, list())
@@ -317,6 +475,7 @@ class ChannelConfig(SlacktivateConfigSection):
             self,
             users: typing.Union[list, dict],
             groups: typing.List[UserGroupConfig],
+            vars: typing.Optional[typing.Dict[str, str]],
     ) -> typing.List["ChannelConfig"]:
 
         target_users = users
@@ -357,6 +516,7 @@ class ChannelConfig(SlacktivateConfigSection):
             channel_name = slacktivate.input.helpers.render_jinja2(
                 jinja2_pattern=self.get("name"),
                 data=user,
+                vars=vars,
             )
 
             subchannel_users[channel_name] = subchannel_users.get(channel_name, list())
