@@ -7,6 +7,8 @@ the Slack SCIM API.
 
 import typing
 
+import loguru
+
 import slacktivate.helpers.photo
 import slacktivate.input.config
 import slacktivate.input.helpers
@@ -68,15 +70,25 @@ _users_cache_by_id: typing.Optional[typing.Dict[str, slacktivate.slack.classes.S
 currently logged-in Slack workspace, to speed up queries."""
 
 
-def _refresh_users_cache() -> typing.NoReturn:
+# Logger
+logger = loguru.logger
+
+
+def _refresh_users_cache(index_by_alternate_emails=False) -> typing.NoReturn:
     """
     Refreshes the two global internal caches of users (:py:attr:`_users_cache_by_email`
     and :py:attr:`_users_cache_by_id`) that this module uses to speed up queries
     over users and avoid hitting rate-limiting quotas too frequently.
+
+    :param index_by_alternate_emails: Flag to indicate whether all the emails of
+        the users (i.e., including the alternate ones) should be indexed in the cache,
+        or only the primary email.
+    :type index_by_alternate_emails: :py:class:`bool`
     """
     global _users_cache_by_email, _users_cache_by_id
 
     result = slacktivate.slack.clients.scim().search_users(count=MAX_USER_LIMIT)
+    logger.debug("Retrieved {} users from Slack SCIM API.", len(result.resources))
 
     _users_cache_by_email = dict()
     _users_cache_by_id = dict()
@@ -88,9 +100,16 @@ def _refresh_users_cache() -> typing.NoReturn:
         if user is None or not user.exists:
             continue
 
+        logger.debug("Caching user {} ({}) active: {}...", user.email, user.id, user.active)
+
         # index by primary email
         _users_cache_by_email[user.email] = user
 
+        # index by secondary emails
+        if index_by_alternate_emails:
+            for email in user.emails:
+                _users_cache_by_email[email] = user
+
         # index by id
         _users_cache_by_id[user.id] = user
 
@@ -482,18 +501,37 @@ def users_ensure(
     # get emails of cached users
     # NOTE: if needed to deal with alternate emails, would be here
     active_user_emails = [
-        user_email
-        for user_email in _iterate_emails()
+        user_email.lower()
+        for user_email, user in _iterate_email_and_user()
+        if user.active
     ]
+    existing_user_emails = [
+        user_email.lower()
+        for user_email, user in _iterate_email_and_user()
+    ]
+    logger.debug("Active user emails (count: {}): {}", len(active_user_emails), active_user_emails)
+    logger.debug("Existing user emails (count: {}): {}", len(existing_user_emails), existing_user_emails)
 
     users_to_create = {}
 
     # iterate over all users in config
     for user_email, user_attributes in config.users.items():
 
+        logger.info("Considering {} with {}", user_email, user_attributes)
+
         # user already exists
-        if user_email.lower() in active_user_emails:
+        if user_email.lower() in existing_user_emails:
+            logger.info("=> {} in EXISTING users", user_email)
+
+            # if user is not active, warn
+            if user_email.lower() not in active_user_emails:
+                logger.info("=> {} in ACTIVE users", user_email)
+                logger.warning("User {} is not active, but exists: Need to use `synchronize` to reactivate", user_email)
+
             continue
+        
+        # user does not exist, so create
+        logger.info("=> {} does not exist and will be created", user_email)
 
         users_to_create[user_email] = user_attributes
 
@@ -519,9 +557,19 @@ def users_ensure(
             include_fields=True,
         )
 
-        new_user = slacktivate.slack.methods.user_create(
-            attributes=processed_attributes,
-        )
+        try:
+            new_user = slacktivate.slack.methods.user_create(
+                attributes=processed_attributes,
+            )
+        except slacktivate.slack.clients.SCIMApiError as exc:
+            logger.error(
+                "Failed to create user {email} [{exists}] with attributes {attributes}: {exc}",
+                email=user_email,
+                exists=user_email.lower() in existing_user_emails,
+                attributes=user_attributes,
+                exc=exc,
+            )
+            continue
 
         users_created[user_email] = new_user
 
