@@ -11,6 +11,7 @@ the logged-in Slack workspace.
 import itertools
 import typing
 
+import loguru
 import slack
 import slack.errors
 import slack_scim
@@ -58,6 +59,9 @@ for more information).
 _custom_fields_by_label: typing.Optional[typing.Dict[str, dict]] = None
 
 
+logger = loguru.logger
+
+
 @slacktivate.slack.retry.slack_retry
 def user_patch(
         user: slacktivate.slack.classes.SlackUserTypes,
@@ -86,11 +90,20 @@ def user_patch(
     if user is None:
         return
 
+    result = None
     with slacktivate.slack.clients.managed_scim() as scim:
-        result = scim.patch_user(
-            id=user.id,
-            user=changes,
-        )
+        try:
+            result = scim.patch_user(
+                id=user.id,
+                user=changes,
+            )
+        except slack_scim.v1.errors.SCIMApiError as exc:
+            logger.error(
+                "Failed to patch user {user} with changes {changes}: {exc}",
+                user=user,
+                changes=changes,
+                exc=exc,
+            )
 
     if result is not None:
         return slacktivate.slack.classes.to_slack_user(result)
@@ -172,6 +185,7 @@ def user_create(
     #     "emails" in attributes
     # )
 
+    result = None
     with slacktivate.slack.clients.managed_scim() as scim:
         result = scim.create_user(
             user=attributes,
@@ -531,6 +545,7 @@ def user_profile_set(
     if user is None:
         return
 
+    result = None
     with slacktivate.slack.clients.managed_api() as slack_client:
         result = slack_client.users_profile_set(
             user=user.id,
@@ -571,6 +586,7 @@ def group_create(
     if grp.exists:
         return grp
 
+    result = None
     with slacktivate.slack.clients.managed_scim() as scim:
         new_grp = slack_scim.Group.from_dict({
             "displayName": display_name
